package com.ibm.oti.shared;

/*
 * Licensed Materials - Property of IBM,
 *     Copyright IBM Corp. 1998, 2010  All Rights Reserved
 */

import java.net.URL;

/**
 * Implementation of SharedClassURLHelper.
 * <p>
 * @author		OTI
 * @version		initial
 *
 * @see SharedClassURLHelper
 * @see SharedClassHelperFactory
 * @see SharedClassAbstractHelper
 */
public class SharedClassURLHelperImpl extends SharedClassAbstractHelper implements SharedClassURLHelper {
	/* Not public - should only be created by factory */
	private boolean minimizeUpdateChecks;

	// This field is examined using the CDS adaptor
	public static final boolean MINIMIZE_ENABLED = true;

	SharedClassURLHelperImpl(ClassLoader loader, int id, boolean canFind, boolean canStore) {
		initialize(loader, id, canFind, canStore);
		initializeShareableClassloader(loader);
	}

	private native boolean findSharedClassImpl3(int id, String partition, String className, ClassLoader loader, URL url,
			boolean doFind, boolean doStore, byte[] romClassCookie, boolean minimizeUpdateChecks);

	private native boolean storeSharedClassImpl3(int id, String partition, ClassLoader loader, URL url, Class clazz, boolean minimizeUpdateChecks, byte[] nativeFlags);

	public boolean setMinimizeUpdateChecks() {
		minimizeUpdateChecks = true;
		return true;
	}

	public byte[] findSharedClass(URL path, String className) {
		return findSharedClass(null, path, className);
	}

	public synchronized byte[] findSharedClass(String partition, URL path, String className) {
		if (!canFind) {
			return null;
		}
		if (path==null) {
			printVerboseError("Cannot call findSharedClass with null URL. Returning null.");
			return null;
		}
		if (className==null) {
			printVerboseError("Cannot call findSharedClass with null class name. Returning null.");
			return null;
		}
		SharedClassFilter theFilter = getSharingFilter();
		boolean doFind, doStore;
		if (theFilter!=null) {
			doFind = theFilter.acceptFind(className);
			/* Don't invoke the store filter if the cache is full */
			if (nativeFlags[CACHE_FULL_FLAG] == 0) {
				doStore = theFilter.acceptStore(className);
			} else {
				doStore = true;
			}
		} else {
			doFind = true;
			doStore = true;
		}
		URL convertedPath = convertJarURL(path);
		/* Too expensive to touch every URL to see if it exists. Leave this check to the cache. */
		if (!validateURL(convertedPath, false)) {
			return null;
		}
		byte[] romClassCookie = new byte[ROMCLASS_COOKIE_SIZE];
		boolean found = findSharedClassImpl3(this.id, partition, className, getClassLoader(), convertedPath, doFind, doStore, romClassCookie, minimizeUpdateChecks);
		if (!found) {
			return null;
		}
		return romClassCookie;
	}

	public boolean storeSharedClass(URL path, Class clazz) {
		return storeSharedClass(null, path, clazz);
	}

	public synchronized boolean storeSharedClass(String partition, URL path, Class clazz) {
		if (!canStore) {
			return false;
		}
		if (path==null) {
			printVerboseError("Cannot call storeSharedClass with null URL. Returning false.");
			return false;
		}
		if (clazz==null) {
			printVerboseError("Cannot call storeSharedClass with null Class. Returning false.");
			return false;
		}
		URL convertedPath = convertJarURL(path);
		/* Too expensive to touch every URL to see if it exists. Leave this check to the cache. */
		if (!validateURL(convertedPath, false)) {
			return false;
		}
		ClassLoader actualLoader = getClassLoader();
		if (!validateClassLoader(actualLoader, clazz)) {
			/* Attempt to call storeSharedClass with class defined by a different classloader */
			return false;
		}
		return storeSharedClassImpl3(this.id, partition, actualLoader, convertedPath, clazz, minimizeUpdateChecks, nativeFlags);
	}

	String getHelperType() {
		return "SharedClassURLHelper";
	}
}
