/*
 * Licensed Materials - Property of IBM,
 *     Copyright IBM Corp. 2009, 2010  All Rights Reserved
 */

package com.ibm.tools.attach.javaSE;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;

/**
 *
 * Manage socket communications between an attacher and a target VM.
 *
 */
class AttachmentConnection {

	/**
	 * read until null byte found
	 * @param channel input stream
	 * @return array of bytes, not including null termination
	 * @throws IOException if file closed before null found
	 */

	static byte[] streamReceiveBytes(InputStream channel) throws IOException  {
		final int bufferLen = 100; /* most messages are short */
		byte [] msgBuff = new byte[bufferLen];
		ByteArrayOutputStream msg = new ByteArrayOutputStream(bufferLen);

		if (null == channel) {
			throw new IOException();
		}
		boolean done = false;
		while (!done) {
			int	nRead = channel.read(msgBuff, 0, bufferLen);
			if (nRead > 0) {
				if (msgBuff[nRead-1] == 0) { /* messages are terminated by a null */
					done = true;
					nRead--; /* strip the terminating null */
				}
				msg.write(msgBuff, 0, nRead);
			} else {
				throw new IOException("input stream closed"); /* premature close of the socket */ //$NON-NLS-1$
			}
		}
		return msg.toByteArray();
	}

	/**
	 * writes the data with a null termination
	 * @param channel output channel
	 * @param data data to write
	 * @throws IOException if channel not open
	 */
	static void streamSend(OutputStream channel, String data) throws IOException {
		IPC.logMessage("streamSend "+data); //$NON-NLS-1$
		if (null == channel) {
			throw new IOException();
		}
		channel.write(data.getBytes("UTF8")); //$NON-NLS-1$
		channel.write(0);
		channel.flush();
	}

	/**
	 * read until null byte found
	 * @param channel input stream
	 * @return string with entire message
	 * @throws IOException if file closed before null found
	 */
	static String streamReceiveString(InputStream channel) throws IOException  {
		byte[] msgBuff = streamReceiveBytes(channel);
		String msg = (new BufferedReader(new InputStreamReader(new ByteArrayInputStream(msgBuff),  "UTF8"))).readLine(); //$NON-NLS-1$
		return msg;
	}

}
